/*****************************************************************************
 * Deobjectivizer Companion Framework                                        *
 * Copyright (C) 2010 Charles E. Davis                                       *
 *                                                                           *
 * This library is free software...                                          *
 *                                                                           *
 * Deobjectivizer.h - Primary public header for the Deobjectivizer frame-    *
 *                    work.                                                  *
 *****************************************************************************/

#ifndef DEOBJECTIVIZER_DEOBJECTIVIZER_H
#define DEOBJECTIVIZER_DEOBJECTIVIZER_H 1

/*
 * The purpose of this framework                                             
 *
 * The Deobjectivizer tool allows you to generate a pure procedural C binding
 * to an Objective-C framework. But often, this is not enough. Interacting
 * with many Objective-C frameworks requires the client to implement its own
 * objects.
 * 
 * This is where the Deobjectivizer companion framework comes in. The
 * Objective-C standard library (libobjc) provides many support routines
 * which are used to implement the object system. Sending a message, for
 * example, is handled by a function called objc_msgSend().
 *
 * The Deobjectivizer tool generates stub functions that send messages to an
 * object, so you don't have to call objc_msgSend() yourself. The
 * Deobjectivizer framework extends this concept, allowing you to easily
 * create your own Objective-C classes--without writing a single line of
 * Objective-C code.
 *
 * The Deobjectivizer framework also reexports many of the functions provided
 * by libobjc, allowing you to call these functions without needing to link to
 * both Deobjectivizer and libobjc.
 */

#define BOOL ObjBool
#include <objc/objc.h>
#include <objc/message.h>
#include <objc/runtime.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct obj_method {
    const char* name;
    IMP imp;
    const char* types;
} ObjMethod;

typedef struct obj_class {
    const char* name;
    const char* supername;
    const char** protocols;
    ObjMethod* methods;
} ObjClass;

/* Class functions */
Class ObjCRegisterClass(ObjClass* cls);
void ObjCUnregisterClass(ObjClass* cls);

/* Object functions */
void* ObjCGetContext(id obj);
void ObjCSetContext(id obj, void* ctx);

#ifdef __cplusplus
}
#endif

#endif /* DEOBJECTIVIZER_DEOBJECTIVIZER_H */
