// Deobjectivizer - The program that makes C bindings to ObjC frameworks
// Copyright (C) 2010 Charles E. Davis

// This program is free software...

// DeobjectivizerAction.cpp - Implements the DeobjectivizerAction class.

#include "DeobjectivizerAction.h"
#include <llvm/Support/raw_ostream.h>
#include <llvm/Support/PathV2.h>
#include <llvm/Support/FileSystem.h>
#include <clang/Basic/Diagnostic.h>
#include <clang/Frontend/CompilerInstance.h>
#include "DeobjectivizerInvocation.h"

clang::ASTConsumer* CreateDeobjectivizer(const std::string& fwk_name,
                                         const std::string& in_file,
                                         llvm::raw_ostream* os,
                                         llvm::raw_ostream* c_os,
                                         clang::Diagnostic& diag,
                                         const clang::LangOptions& lopts,
                                         bool silence_warn);

clang::ASTConsumer* DeobjectivizerAction::CreateASTConsumer(
    clang::CompilerInstance& CI, llvm::StringRef in_file) {
    llvm::raw_ostream* OS,* C_OS;
    llvm::SmallString<256> hfile;
    llvm::SmallString<32> stem(llvm::sys::path::stem(in_file));
    llvm::sys::fs::current_path(hfile);
    llvm::SmallString<256> cfile(hfile.begin(), hfile.end());
    llvm::sys::path::append(hfile, llvm::Twine(stem)+".h");
    llvm::sys::path::append(cfile, llvm::Twine(stem)+".c");
    DeobjectivizerInvocation& DI =
        static_cast<DeobjectivizerInvocation&>(CI.getInvocation());
    if ((OS = CI.createOutputFile(hfile, false)) &&
        (C_OS = CI.createOutputFile(cfile, false))) {
        return CreateDeobjectivizer(DI.getFrameworkName(), in_file, OS, C_OS,
                                    CI.getDiagnostics(), CI.getLangOpts(),
                                    CI.getDiagnosticOpts().NoRewriteMacros);
    }
    return NULL;
}
