/*****************************************************************************
 * Deobjectivizer Companion Framework                                        *
 * Copyright (C) 2010 Charles E. Davis                                       *
 *                                                                           *
 * This library is free software...                                          *
 *                                                                           *
 * DeobjectivizerFwk.c - Implementation file for the Deobjectivizer frame-   *
 *                       work.                                               *
 *****************************************************************************/

#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "Deobjectivizer.h"

Class ObjCRegisterClass(ObjClass* cls) {
    if(!cls || !cls->name) return Nil;
    Class sc = (Class)objc_getClass(cls->supername);
    Class c = objc_allocateClassPair(sc, cls->name, 0);
    if(c == Nil) return Nil;
    /* Add protocols (if applicable) */
    if (cls->protocols) {
        int i = 0;
        while(cls->protocols[i]) {
            Protocol* p = objc_getProtocol(cls->protocols[i]);
            if (p)
                class_addProtocol(c, p);
        }
    }
    /* Add methods (if applicable) */
    if (cls->methods) {
        int i = 0;
        while(cls->methods[i].name) {
            SEL sel = sel_registerName(cls->methods[i].name);
            class_addMethod(c, sel, cls->methods[i].imp,
                            cls->methods[i].types);
        }
    }
    /* Add the one instance variable */
    class_addIvar(c, "_context", sizeof(void *), sizeof(void *), "^v");
    objc_registerClassPair(c);
    return c;
}

void ObjCUnregisterClass(ObjClass* cls) {
    if (!cls || !cls->name) return;
    objc_disposeClassPair((Class)objc_getClass(cls->name));
}

void* ObjCGetContext(id obj) {
    void* ctx;
    object_getInstanceVariable(obj, "_context", &ctx);
    return ctx;
}

void ObjCSetContext(id obj, void* ctx)
{
	object_setInstanceVariable(obj, "_context", ctx);
}
